@@ -1,6 +1,7 @@
 import os
 import shlex
 import shutil
+from unittest.mock import patch
 
 import boto3
 import pytest
@@ -578,6 +579,9 @@ def test_sharded_dataset() -> None:
     assert dataset.hash == "3394197d206ea0ef46795131b98f86c52ab9a508"
     assert dataset2.hash == "f7cd6b0188ff900f0ca00ea0b937a91d70a3e67a"
 
+    # Check type registry uses the correct hash function.
+    assert dataset.hash == get_type_registry().get_hash(dataset)
+
     file3.remove()
     assert dataset.is_valid()
     assert not dataset2.is_valid()
@@ -922,3 +926,42 @@ def test_copy_file_dir() -> None:
 
     redun.file.copy_file("src", dest_dir.path, recursive=True)
     assert {file.path for file in dest_dir} == {"dest/a.txt", "dest/b/b.txt"}
+
+
+def test_non_existent_file_error() -> None:
+    """
+    File.open should raise descriptive error when path does not exist.
+    """
+
+    with pytest.raises(
+        FileNotFoundError, match=r"\[Errno 2\] No such file or directory\. not_exist_file"
+    ):
+        File("not_exist_file").open()
+
+
+def test_file_error() -> None:
+    """
+    File.open should raise descriptive error when there is a low-level error.
+    """
+
+    def file_open(*args):
+        raise OSError(100, "My OS error")
+
+    fs = get_filesystem(proto="local")
+    with patch.object(fs, "_open", file_open):
+        with pytest.raises(OSError, match=r"\[Errno 100\] My OS error\. bad_file"):
+            File("bad_file").open()
+
+
+# Root user will still be able to write to a readonly file so skip this test if we are running
+# as root (which will be the case on codebuild runs).
+@use_tempdir
+@pytest.mark.skipif(os.getuid() == 0, reason="Running as root which can write to readonly files")
+def test_readonly_file_error() -> None:
+    """
+    File.open should raise descriptive error when trying to open a readonly file in write mode.
+    """
+    File("readonly_file").write("hi")
+    os.system("chmod 400 readonly_file")
+    with pytest.raises(PermissionError, match=r"\[Errno 13\] Permission denied\. readonly_file"):
+        File("readonly_file").open("w")
